<?php

declare(strict_types = 1);

namespace Drupal\state_machine_permissions;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Dynamic permissions provider.
 */
class StateMachinePermissions implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The permission helper service.
   *
   * @var \Drupal\state_machine_permissions\StateMachinePermissionsHelperInterface
   */
  protected $permissionsHelper;

  /**
   * Constructs a state machine permissions object.
   *
   * @param \Drupal\state_machine_permissions\StateMachinePermissionsHelperInterface $permissionsHelper
   *   The permission helper service.
   */
  public function __construct(StateMachinePermissionsHelperInterface $permissionsHelper) {
    $this->permissionsHelper = $permissionsHelper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('state_machine_permissions.helper'));
  }

  /**
   * Returns an array of permissions related to state machine.
   *
   * @return array
   *   Dynamically generated permissions.
   */
  public function stateMachinePermissions(): array {
    $permissions = [];
    $state_field_map = $this->permissionsHelper->getStateFieldMap();
    if (empty($state_field_map)) {
      return $permissions;
    }

    foreach ($state_field_map as $entity_type_id => $bundles) {
      foreach ($bundles as $bundle => $fields) {
        $permissions += $this->generateWorkflowPermissionsForBundle($entity_type_id, $bundle, $fields);
      }
    }
    return $permissions;
  }

  /**
   * Returns an array of permissions per transmission for the given bundle.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   * @param string $bundle
   *   The entity bundle.
   * @param array $field_names
   *   An array of state field names.
   *
   * @return array
   *   An array of permissions keyed by the permission id where each entry
   *   contains the 'label' key and the description.
   */
  protected function generateWorkflowPermissionsForBundle(string $entity_type_id, string $bundle, array $field_names): array {
    $workflows = $this->permissionsHelper->getPossibleWorkflowsForBundle($entity_type_id, $bundle, $field_names);

    $permissions = [];
    foreach ($workflows as $workflow) {
      $transitions = $workflow->getTransitions();
      foreach ($transitions as $transition) {
        $to_state = $transition->getToState();
        foreach ($transition->getFromStates() as $from_state) {
          foreach (['own' => FALSE, 'any' => TRUE] as $key => $permission) {
            $permissions[StateMachinePermissionStringConstructor::constructTransitionPermission($entity_type_id, $bundle, $workflow, $from_state->getId(), $to_state->getId(), $permission)] = [
              'title' => $this->t('@workflow_label (:workflow_id): Transition from @from_state to @to_state - :key :bundle :entity_type_id entity', [
                '@workflow_label' => $workflow->getLabel(),
                ':workflow_id' => $workflow->getId(),
                '@from_state' => $from_state->getLabel(),
                '@to_state' => $to_state->getLabel(),
                ':key' => $key,
                ':bundle' => $bundle,
                ':entity_type_id' => $entity_type_id,
              ]),
            ];
            $permissions[StateMachinePermissionStringConstructor::constructGroupStateUpdatePermission($workflow, $from_state->getId(), $to_state->getId())] = [
              'title' => $this->t('@workflow_label (:workflow_id): Transition group from @from_state to @to_state', [
                '@workflow_label' => $workflow->getLabel(),
                ':workflow_id' => $workflow->getId(),
                '@from_state' => $from_state->getLabel(),
                '@to_state' => $to_state->getLabel(),
              ]),
            ];
          }
        }
      }
    }
    return $permissions;
  }

}
